module Charset(stringToBytes, bytesToString, charsetSetup, validCharsets, csHPutStr) where


import UTF8
import System
import Char
import Word(Word8)
import IOExts
import List
import EIO

{-# NOINLINE stb_r #-}
{-# NOINLINE bts_r #-}
stb_r = unsafePerformIO $ newIORef toLatin1 
bts_r = unsafePerformIO $ newIORef fromSingleByte

{-# INLINE stringToBytes #-}
stringToBytes :: String -> [Word8]
stringToBytes s = f s where
    f = unsafePerformIO $ readIORef stb_r

{-# INLINE bytesToString #-}
bytesToString :: [Word8] -> String
bytesToString s = f s where
    f = unsafePerformIO $ readIORef bts_r

validCharsets = map fst charMap

charMap = [
    (["UTF8"],(toUTF,fromUTF)), 
    (["ASCII"],(toAscii,fromSingleByte)), 
    (["LATIN1","ISO88591"],(toLatin1,fromSingleByte)) ]


normalize s = map toUpper . filter isAlphaNum $ s 

charsetSetup :: Maybe String -> IO ()
charsetSetup (Just s) = case [x| (al ,x) <- charMap, normalize s `elem` al ] of
    ((a,b):_) -> writeIORef stb_r a >> writeIORef bts_r b
    _ -> return ()
    
    
charsetSetup Nothing = do
    e <- catch (fmap Just $ getEnv "LC_CTYPE") (\_ -> catch (fmap Just $ getEnv "LANG") (\_ -> return Nothing))
    case e of
	Nothing -> return ()
	Just e -> case [x| (al ,x) <- charMap, any ((normalize e) `isSuffixOf`) al ] of
	    ((a,b):_) -> writeIORef stb_r a >> writeIORef bts_r b
	    _ -> return ()

    


toAscii :: String -> [Word8]
toAscii s = map f s where
    f x | ord x > 127 = fromIntegral (ord '?')
    f x = fromIntegral $ ord x

toLatin1 :: String -> [Word8]
toLatin1 s = map f s where
    f x | ord x > 255 = fromIntegral (ord '?')
    f x = fromIntegral $ ord x

fromSingleByte :: [Word8] -> String
fromSingleByte = map (chr . fromIntegral)



csHPutStr h s = do
    conv <- readIORef stb_r
    putRaw h (conv s)
    

